import React, {
  createContext,
  useCallback,
  useContext,
  useRef,
  useState,
} from "react";
import { IMessageData, ISocketUser } from "../types";
import { SocketClient, ReplyMessagePayload } from "./SocketClient";

type SocketContextType = {
  connectedUsers: ISocketUser[];
  messageData: IMessageData;
  connect: (user: ISocketUser) => void;
  sendMessage: (toId: string, message: string) => void;
  readMessage: (toId: string, msgId: string) => void;
  readMessages: (id: string) => void;
};

const SocketContext = createContext<SocketContextType>({
  connectedUsers: [],
  messageData: {},
  connect: () => null,
  sendMessage: () => null,
  readMessage: () => null,
  readMessages: () => null,
});

interface ISocketProvider {
  children: React.ReactNode;
  socketServerUrl: string;
}

export const SocketProvider = ({
  children,
  socketServerUrl,
}: ISocketProvider) => {
  const [connectedUsers, setConnectedUsers] = useState<ISocketUser[]>([]);
  const [messageData, setMessageData] = useState<IMessageData>({});
  const socket = useRef<any>(null);

  const connect = useCallback((user: ISocketUser) => {
    if (user?.name) {
      socket.current = new SocketClient(socketServerUrl);
      socket.current.connect(user?.name!);
      socket.current.listenLiveUsers((liveUsers: ISocketUser[]) => {
        setConnectedUsers(liveUsers);
      });
      socket.current.listenReply(({ fromId, message }: ReplyMessagePayload) => {
        setMessageData((prevMssgs) => {
          return {
            ...prevMssgs,
            [fromId]: [
              ...(prevMssgs[fromId] ?? []),
              {
                id: fromId,
                name: "",
                message,
                unread: true,
              },
            ],
          };
        });
      });
    }
  }, []);

  const sendMessage = useCallback((toId: string, message: string) => {
    if (socket.current) {
      socket.current.sendMessage(toId, message);
      setMessageData((prevMssgs) => {
        return {
          ...prevMssgs,
          [toId]: [
            ...(prevMssgs[toId] ?? []),
            {
              id: toId,
              name: "",
              message,
              unread: false,
            },
          ],
        };
      });
    }
  }, []);

  const readMessages = useCallback((id: string) => {
    if (id) {
      setMessageData((prevMssgs) => {
        return {
          ...prevMssgs,
          [id]: prevMssgs[id]?.map((mssg) => {
            return { ...mssg, unread: false };
          }),
        };
      });
    }
  }, []);

  const readMessage = useCallback((toId: string, msgId: string) => {
    if (toId && msgId) {
      setMessageData((prevMssgs) => {
        return {
          ...prevMssgs,
          [toId]: prevMssgs[toId]?.map((mssg) => {
            if (mssg.unread && mssg.id === msgId) {
              return { ...mssg, unread: false };
            }
            return mssg;
          }),
        };
      });
    }
  }, []);

  return (
    <SocketContext.Provider
      value={{
        connectedUsers,
        messageData,
        connect,
        sendMessage,
        readMessage,
        readMessages,
      }}
    >
      {children}
    </SocketContext.Provider>
  );
};

export const useSocketContext = () => useContext(SocketContext);
